@@ -218,14 +218,28 @@ class TokamakEquilibrium(Equilibrium):
             ),
             value_type=[float, int, NoneType],
         ),
+        start_at_upper_outer=WithMeta(
+            False,
+            doc=(
+                "Start gridding double-null at upper-outer divertor instead of "
+                "lower-inner. Warning: this option was added to enable backward "
+                "compatibility with restart files from simulations in "
+                "upper-disconnected-double-null configuration using grid files from "
+                "the IDL hypnotoad; it is not well tested and not recommended to use."
+            ),
+            value_type=bool,
+        ),
         # Tolerance for positioning points that should be at X-point, but need to be
         # slightly displaced from the null so code can follow Grad(psi).
         # Number between 0. and 1.
         xpoint_offset=WithMeta(
             0.1,
             doc=(
-                "Tolerance for positioning points that should be at X-point, but need "
-                "to be slightly displaced from the null so code can follow Grad(psi)."
+                "Tolerance for placing intial positions for tracing perpendiculars "
+                "that should start exactly at an X-point, but initial positions to be "
+                "slightly displaced from the null so code can follow Grad(psi).  This "
+                "is a numerical fudge factor that may need to be increased for "
+                "low-resolution input equilibria."
             ),
             value_type=float,
             check_all=[is_positive, lambda x: x < 1.0],
@@ -381,7 +395,7 @@ class TokamakEquilibrium(Equilibrium):
         super().__init__(nonorthogonal_settings)
 
         # Print the table of options
-        print(self.user_options.as_table())
+        print(self.user_options.as_table(), flush=True)
 
         if make_regions:
             # Create self.regions
@@ -668,7 +682,7 @@ class TokamakEquilibrium(Equilibrium):
         }
 
         if self.x_points[0].Z < self.o_point.Z:
-            print("Generating a lower single null")
+            print("Generating a lower single null", flush=True)
 
             segments["lower_pf"] = {
                 "nx": self.user_options.nx_pf,
@@ -712,7 +726,7 @@ class TokamakEquilibrium(Equilibrium):
                 ("core", 0, "core", 0),  # Core -> core
             ]
         else:
-            print("Upper Single Null")
+            print("Upper Single Null", flush=True)
 
             # The mesh is still arranged clockwise, meaning that y indexing starts
             # at the outer upper divertor, and ends at the inner upper divertor.
@@ -887,7 +901,7 @@ class TokamakEquilibrium(Equilibrium):
         }
 
         if nx_inter_sep == 0:
-            print("Generating a connected double null")
+            print("Generating a connected double null", flush=True)
 
             # Only use psi of inner separatrix, not the outer one (if it is slightly
             # different)
@@ -976,7 +990,7 @@ class TokamakEquilibrium(Equilibrium):
             )
 
         else:
-            print("Generating a disconnected double null")
+            print("Generating a disconnected double null", flush=True)
 
             # Disconnected double null -> Additional radial segment
             segments["near_sol"] = {
@@ -988,7 +1002,7 @@ class TokamakEquilibrium(Equilibrium):
             }
 
             if self.x_points[0] == lower_x_point:
-                print("Lower double null")
+                print("Lower double null", flush=True)
 
                 inner_lower_segments = ["lower_pf", "near_sol", "inner_sol"]
                 outer_lower_segments = ["lower_pf", "near_sol", "outer_sol"]
@@ -1044,7 +1058,7 @@ class TokamakEquilibrium(Equilibrium):
                 segments["upper_pf"]["nx"] -= nx_inter_sep
 
             else:
-                print("Upper double null")
+                print("Upper double null", flush=True)
                 inner_lower_segments = ["lower_pf", "lower_pf2", "inner_sol"]
                 outer_lower_segments = ["lower_pf", "lower_pf2", "outer_sol"]
 
@@ -1387,18 +1401,31 @@ class TokamakEquilibrium(Equilibrium):
         # BoutMesh generator can use jyseps indices to introduce branch cuts
 
         if "inner_lower_divertor" in region_objects:
-            ordering = [
-                "inner_lower_divertor",
-                # For single null; in double null this will be ignored
-                "core",
-                # For double null; in single null these will be ignored
-                "inner_core",
-                "inner_upper_divertor",
-                "outer_upper_divertor",
-                "outer_core",
-                #
-                "outer_lower_divertor",
-            ]
+            if not self.user_options.start_at_upper_outer:
+                ordering = [
+                    "inner_lower_divertor",
+                    # For single null; in double null this will be ignored
+                    "core",
+                    # For double null; in single null these will be ignored
+                    "inner_core",
+                    "inner_upper_divertor",
+                    "outer_upper_divertor",
+                    "outer_core",
+                    #
+                    "outer_lower_divertor",
+                ]
+            else:
+                # Special case intended for backward compatibility with simulations
+                # using upper-disconnected-double-null IDL-hypnotoad grid files
+                ordering = [
+                    "outer_upper_divertor",
+                    "outer_core",
+                    "outer_lower_divertor",
+                    "inner_lower_divertor",
+                    "core",
+                    "inner_core",
+                    "inner_upper_divertor",
+                ]
         else:
             # Upper single null special case
             ordering = ["outer_upper_divertor", "core", "inner_upper_divertor"]
@@ -1572,7 +1599,7 @@ def read_geqdsk(
         # fpol constant in SOL
         fpol = np.concatenate([fpol, np.full(psiSOL.shape, fpol[-1])])
 
-    return TokamakEquilibrium(
+    result = TokamakEquilibrium(
         R1D,
         Z1D,
         psi2D,
@@ -1584,3 +1611,15 @@ def read_geqdsk(
         settings=settings,
         nonorthogonal_settings=nonorthogonal_settings,
     )
+
+    # Store geqdsk input as a string in the TokamakEquilibrium object so we can save it
+    # in BoutMesh.writeGridFile
+    # reset to beginning of file
+    filehandle.seek(0)
+    # read file as a single string and store in result
+    result.geqdsk_input = filehandle.read()
+    # also save filename, if it exists
+    if hasattr(filehandle, "name"):
+        result.geqdsk_filename = filehandle.name
+
+    return result
