@@ -125,9 +125,16 @@ class HypnotoadGui(QMainWindow, Ui_Hypnotoad):
         self.search_bar.setPlaceholderText("Search options...")
         self.search_bar.textChanged.connect(self.search_options_form)
         self.search_bar.setToolTip(self.search_options_form.__doc__.strip())
-        self.search_bar_completer = QCompleter(
-            list(tokamak.TokamakEquilibrium.user_options_factory.defaults.keys())
+        option_names = (
+            set(BoutMesh.user_options_factory.defaults.keys())
+            .union(set(tokamak.TokamakEquilibrium.user_options_factory.defaults.keys()))
+            .union(
+                set(
+                    tokamak.TokamakEquilibrium.nonorthogonal_options_factory.defaults.keys()  # noqa: E501
+                )
+            )
         )
+        self.search_bar_completer = QCompleter(option_names)
         self.search_bar_completer.setCaseSensitivity(Qt.CaseInsensitive)
         self.search_bar.setCompleter(self.search_bar_completer)
 
@@ -190,8 +197,14 @@ class HypnotoadGui(QMainWindow, Ui_Hypnotoad):
 
         options_to_save = self.options
         if self.gui_options["save_full_yaml"]:
-            options_ = tokamak.TokamakEquilibrium.user_options_factory.create(
-                self.options
+            options_ = BoutMesh.user_options_factory.create(self.options)
+            options_.update(
+                tokamak.TokamakEquilibrium.user_options_factory.create(self.options)
+            )
+            options_.update(
+                tokamak.TokamakEquilibrium.nonorthogonal_options_factory.create(
+                    self.options
+                )
             )
 
             # This converts any numpy types to native Python using the tolist()
@@ -234,7 +247,8 @@ class HypnotoadGui(QMainWindow, Ui_Hypnotoad):
 
         filtered_options = copy.deepcopy(self.options)
 
-        filtered_defaults = dict(
+        filtered_defaults = dict(BoutMesh.user_options_factory.defaults)
+        filtered_defaults.update(
             tokamak.TokamakEquilibrium.user_options_factory.defaults
         )
         filtered_defaults.update(
@@ -243,8 +257,11 @@ class HypnotoadGui(QMainWindow, Ui_Hypnotoad):
 
         # evaluate filtered_defaults using the values in self.options, so that any
         # expressions get evaluated
+        filtered_default_values = dict(
+            BoutMesh.user_options_factory.create(self.options)
+        )
         try:
-            filtered_default_values = dict(
+            filtered_default_values.update(
                 tokamak.TokamakEquilibrium.user_options_factory.create(self.options)
             )
             if not hasattr(self, "eq"):
@@ -344,10 +361,30 @@ class HypnotoadGui(QMainWindow, Ui_Hypnotoad):
         self.statusbar.showMessage("Reading options", 2000)
         options_filename = self.options_file_line_edit.text()
 
+        # Save the existing options in case there is an error loading the options file
+        original_options = self.options
+
         if options_filename:
             with open(options_filename, "r") as f:
                 self.options = yaml.safe_load(f)
 
+        possible_options = (
+            [opt for opt in tokamak.TokamakEquilibrium.user_options_factory.defaults]
+            + [
+                opt
+                for opt in tokamak.TokamakEquilibrium.nonorthogonal_options_factory.defaults  # noqa: E501
+            ]
+            + [opt for opt in BoutMesh.user_options_factory.defaults]
+        )
+        unused_options = [opt for opt in self.options if opt not in possible_options]
+        if unused_options != []:
+            short_filename = pathlib.Path(options_filename).parts[-1]
+            self._popup_error_message(
+                f"Error: There were options in the input file that are not used: "
+                f"{unused_options}. Cannot load {short_filename}."
+            )
+            self.options = original_options
+
         self.options_form.setRowCount(0)
         self.update_options_form()
 
@@ -547,13 +584,13 @@ class Preferences(QDialog, Ui_Preferences):
 
     def accept(self):
 
-        self.parent.gui_options.set(
-            grid_file=self.defaultGridFileNameLineEdit.text(),
-            plot_xlow=self.plotXlowCheckBox.isChecked(),
-            plot_ylow=self.plotYlowCheckBox.isChecked(),
-            plot_corners=self.plotCornersCheckBox.isChecked(),
-            save_full_yaml=self.saveFullYamlCheckBox.isChecked(),
-        )
+        self.parent.gui_options["grid_file"] = self.defaultGridFileNameLineEdit.text()
+        self.parent.gui_options["plot_xlow"] = self.plotXlowCheckBox.isChecked()
+        self.parent.gui_options["plot_ylow"] = self.plotYlowCheckBox.isChecked()
+        self.parent.gui_options["plot_corners"] = self.plotCornersCheckBox.isChecked()
+        self.parent.gui_options[
+            "save_full_yaml"
+        ] = self.saveFullYamlCheckBox.isChecked()
 
         self.parent.plot_grid()
 
